"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConnectionDirectiveTypeDefs = exports.Connection = exports.Edge = exports.PageInfo = void 0;
const utils_1 = require("@graphql-tools/utils");
exports.PageInfo = `
  type PageInfo {
    hasNextPage: Boolean!
    hasPreviousPage: Boolean!
    startCursor: String
    endCursor: String
  }`;
const Edge = (typeName, args) => {
    const cacheControl = composeCacheContolDirective(args);
    return `
  type ${typeName}Edge {
    cursor: String!
    node: ${typeName} ${cacheControl}
  }
`;
};
exports.Edge = Edge;
const Connection = (typeName, args) => {
    const cacheControl = composeCacheContolDirective(args);
    return `
  type ${typeName}Connection ${cacheControl} {
    totalCount: Int!
    edges: [${typeName}Edge] ${cacheControl}
    pageInfo: PageInfo! ${cacheControl}
  }`;
};
exports.Connection = Connection;
const composeCacheContolDirective = (args) => {
    if (args === undefined)
        return '@cacheControl(inheritMaxAge: true)';
    if (args === null || args === void 0 ? void 0 : args.hasOwnProperty('maxAge'))
        return `@cacheControl(maxAge: ${args.maxAge})`;
    return '';
};
const getConnectionDirectiveTypeDefs = (directiveName) => {
    return `directive @${directiveName || 'connection'} on FIELD_DEFINITION`;
};
exports.getConnectionDirectiveTypeDefs = getConnectionDirectiveTypeDefs;
// Find connection directives
exports.default = (directiveName, cacheControlOptions) => {
    var _a;
    const name = directiveName || 'connection';
    const objectTypes = {};
    const useCacheControl = (_a = cacheControlOptions === null || cacheControlOptions === void 0 ? void 0 : cacheControlOptions.enable) !== null && _a !== void 0 ? _a : Boolean(cacheControlOptions);
    const useApolloInheritMaxAge = cacheControlOptions === null || cacheControlOptions === void 0 ? void 0 : cacheControlOptions.apollo;
    return {
        connectionObjects: () => objectTypes,
        connectionDirectiveTypeDefs: (0, exports.getConnectionDirectiveTypeDefs)(name),
        connectionDirectiveTransformer: (schema) => {
            return (0, utils_1.mapSchema)(schema, {
                [utils_1.MapperKind.OBJECT_FIELD]: (fieldConfig) => {
                    var _a, _b, _c;
                    const directives = (0, utils_1.getDirectives)(schema, fieldConfig);
                    const typeName = (_b = (_a = fieldConfig.astNode.type) === null || _a === void 0 ? void 0 : _a.name) === null || _b === void 0 ? void 0 : _b.value;
                    if (!typeName)
                        return fieldConfig;
                    const connectionDirective = directives.find((value) => value.name === name);
                    if (!connectionDirective)
                        return fieldConfig;
                    if (!useCacheControl) {
                        // Skip cacheContol directives
                        if (!objectTypes.hasOwnProperty(typeName))
                            objectTypes[typeName] = {};
                        return fieldConfig;
                    }
                    const cacheControlDirective = directives.find((value) => value.name === 'cacheControl');
                    if (useApolloInheritMaxAge) {
                        // Rely on inheritMaxAge instead of calculating maxAge (Apollo v3+)
                        if (!objectTypes.hasOwnProperty(typeName))
                            objectTypes[typeName] = undefined;
                        return fieldConfig;
                    }
                    // Fallback to legacy cacheControl calculation (when inheritMaxAge is not supported)
                    const args = (cacheControlDirective === null || cacheControlDirective === void 0 ? void 0 : cacheControlDirective.args) || {};
                    if (!objectTypes.hasOwnProperty(typeName))
                        objectTypes[typeName] = args;
                    else if ((args === null || args === void 0 ? void 0 : args.maxAge) > ((_c = objectTypes[typeName]) === null || _c === void 0 ? void 0 : _c.maxAge))
                        objectTypes[typeName].maxAge = args.maxAge;
                    return fieldConfig;
                },
            });
        },
    };
};
