"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createConnectionTypes = void 0;
const graphql_1 = require("graphql");
const utils_1 = require("@graphql-tools/utils");
const schema_1 = require("@graphql-tools/schema");
const merge_1 = require("@graphql-tools/merge");
const connection_1 = __importStar(require("./connection"));
const createConnectionTypes = (objectTypes) => {
    return [
        !objectTypes.hasOwnProperty('PageInfo') ? connection_1.PageInfo : undefined,
        ...Object.entries(objectTypes).reduce((acc, [typeName, args]) => {
            if (!!objectTypes.hasOwnProperty(`${typeName}Edge`))
                throw new Error(`${typeName}Edge already exists.`);
            if (!!objectTypes.hasOwnProperty(`${typeName}Connection`))
                throw new Error(`${typeName}Connection already exists.`);
            return [...acc, (0, connection_1.Edge)(typeName, args), (0, connection_1.Connection)(typeName, args)];
        }, []),
    ].filter((node) => !!node);
};
exports.createConnectionTypes = createConnectionTypes;
const getConnectionArgs = () => {
    const argumentsAST = [
        {
            kind: graphql_1.Kind.INPUT_VALUE_DEFINITION,
            description: undefined,
            name: {
                kind: graphql_1.Kind.NAME,
                value: 'after',
            },
            type: {
                kind: graphql_1.Kind.NAMED_TYPE,
                name: {
                    kind: graphql_1.Kind.NAME,
                    value: 'String',
                },
            },
            defaultValue: undefined,
            directives: [],
        },
        {
            kind: graphql_1.Kind.INPUT_VALUE_DEFINITION,
            description: undefined,
            name: {
                kind: graphql_1.Kind.NAME,
                value: 'first',
            },
            type: {
                kind: graphql_1.Kind.NAMED_TYPE,
                name: {
                    kind: graphql_1.Kind.NAME,
                    value: 'Int',
                },
            },
            defaultValue: undefined,
            directives: [],
        },
        {
            kind: graphql_1.Kind.INPUT_VALUE_DEFINITION,
            description: undefined,
            name: {
                kind: graphql_1.Kind.NAME,
                value: 'before',
            },
            type: {
                kind: graphql_1.Kind.NAMED_TYPE,
                name: {
                    kind: graphql_1.Kind.NAME,
                    value: 'String',
                },
            },
            defaultValue: undefined,
            directives: [],
        },
        {
            kind: graphql_1.Kind.INPUT_VALUE_DEFINITION,
            description: undefined,
            name: {
                kind: graphql_1.Kind.NAME,
                value: 'last',
            },
            type: {
                kind: graphql_1.Kind.NAMED_TYPE,
                name: {
                    kind: graphql_1.Kind.NAME,
                    value: 'Int',
                },
            },
            defaultValue: undefined,
            directives: [],
        },
    ];
    const args = argumentsAST.reduce((acc, value) => {
        const name = value?.name?.value;
        const type = value?.type?.name?.value;
        if (!name || !type)
            return acc;
        acc[name] = {
            description: undefined,
            type: type === 'String' ? graphql_1.GraphQLString : graphql_1.GraphQLInt,
            defaultValue: undefined,
            deprecationReason: undefined,
            extensions: undefined,
            astNode: value,
        };
        return acc;
    }, {});
    return { args, argumentsAST };
};
const schemaBuilder = (typeDefs) => {
    return (0, schema_1.makeExecutableSchema)({
        typeDefs,
        resolverValidationOptions: { requireResolversForResolveType: 'ignore' },
        assumeValidSDL: true,
    });
};
exports.default = ({ typeDefs, overrideDirectiveName, cacheControl = { enable: true }, }) => {
    const directiveName = overrideDirectiveName || 'connection';
    const { connectionObjects, connectionDirectiveTypeDefs, connectionDirectiveTransformer, } = (0, connection_1.default)(directiveName, cacheControl);
    // Run transformer to get all types with connection directives
    connectionDirectiveTransformer(schemaBuilder(typeDefs));
    const objectTypeList = connectionObjects();
    const connectionTypeDefs = (0, exports.createConnectionTypes)(objectTypeList);
    const schema = schemaBuilder((0, merge_1.mergeTypeDefs)([typeDefs, connectionTypeDefs, connectionDirectiveTypeDefs]));
    const schemaMapper = (fieldConfig) => {
        const connectionDirectiveType = (0, utils_1.getDirective)(schema, fieldConfig, directiveName)?.[0];
        if (!connectionDirectiveType)
            return fieldConfig;
        const typeName = fieldConfig.type?.toString()?.replace('!', 'NonNull');
        const targetType = schema.getType(`${typeName}Connection`);
        if (!typeName || !targetType)
            return fieldConfig;
        fieldConfig.type = targetType;
        fieldConfig.astNode.type = targetType.astNode;
        const { args, argumentsAST } = getConnectionArgs();
        fieldConfig.args = { ...fieldConfig.args, ...args };
        fieldConfig.astNode.arguments = [
            ...(fieldConfig.astNode.arguments || []),
            ...argumentsAST,
        ];
        fieldConfig.astNode.directives = fieldConfig.astNode.directives.filter((value) => !value?.name?.value?.includes(directiveName));
        return fieldConfig;
    };
    // Transform types with a connection directive into a connection type
    return (0, utils_1.printSchemaWithDirectives)((0, utils_1.mapSchema)(schema, {
        [utils_1.MapperKind.INTERFACE_FIELD]: schemaMapper,
        [utils_1.MapperKind.OBJECT_FIELD]: schemaMapper,
    }));
};
